@@ -282,7 +282,7 @@ static int php_http_curle_raw_callback(CURL *ch, curl_infotype type, char *data,
 			} else if (php_memnstr(data, ZEND_STRL("Operation timed out"), data + length)) {
 				h->progress.info = "timeout";
 			} else {
-#if PHP_DEBUG
+#if 0
 				h->progress.info = data;
 				data[length - 1] = '\0';
 #endif
@@ -785,8 +785,6 @@ static void php_http_curlm_timer_callback(CURLM *multi, long timeout_ms, void *t
 
 			if (!event_initialized(curl->timeout)) {
 				event_assign(curl->timeout, curl->evbase, CURL_SOCKET_TIMEOUT, 0, php_http_curlm_timeout_callback, context);
-			} else if (event_pending(curl->timeout, EV_TIMEOUT, NULL)) {
-				event_del(curl->timeout);
 			}
 
 			timeout.tv_sec = timeout_ms / 1000;
@@ -907,7 +905,7 @@ static STATUS php_http_curle_option_set_lastmodified(php_http_option_t *opt, zva
 				return FAILURE;
 			}
 		} else {
-			if (CURLE_OK != curl_easy_setopt(ch, CURLOPT_TIMEVALUE, (long) PHP_HTTP_G->env.request.time + Z_LVAL_P(val))) {
+			if (CURLE_OK != curl_easy_setopt(ch, CURLOPT_TIMEVALUE, (long) sapi_get_request_time(TSRMLS_C) + Z_LVAL_P(val))) {
 				return FAILURE;
 			}
 		}
@@ -1600,7 +1598,7 @@ static STATUS php_http_client_curl_handler_prepare(php_http_client_curl_handler_
 	if (storage->url) {
 		pefree(storage->url, 1);
 	}
-	storage->url = pestrdup(PHP_HTTP_INFO(msg).request.url, 1);
+	php_http_url_to_string(PHP_HTTP_INFO(msg).request.url, &storage->url, NULL, 1);
 	curl_easy_setopt(curl->handle, CURLOPT_URL, storage->url);
 
 	/* request method */
@@ -1786,38 +1784,29 @@ static void queue_dtor(php_http_client_enqueue_t *e)
 	php_http_client_curl_handler_dtor(handler);
 }
 
-static php_resource_factory_t *create_rf(const char *url TSRMLS_DC)
+static php_resource_factory_t *create_rf(php_http_url_t *url TSRMLS_DC)
 {
-	php_url *purl;
+	php_persistent_handle_factory_t *pf;
 	php_resource_factory_t *rf = NULL;
+	char *id_str = NULL;
+	size_t id_len;
 
-	if (!url || !*url) {
+	if (!url || (!url->host && !url->path)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot request empty URL");
 		return NULL;
 	}
 
-	purl = php_url_parse(url);
+	id_len = spprintf(&id_str, 0, "%s:%d", STR_PTR(url->host), url->port ? url->port : 80);
 
-	if (!purl) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not parse URL '%s'", url);
-		return NULL;
+	pf = php_persistent_handle_concede(NULL, ZEND_STRL("http\\Client\\Curl\\Request"), id_str, id_len, NULL, NULL TSRMLS_CC);
+	if (pf) {
+		rf = php_resource_factory_init(NULL, php_persistent_handle_get_resource_factory_ops(), pf, (void (*)(void*)) php_persistent_handle_abandon);
 	} else {
-		char *id_str = NULL;
-		size_t id_len = spprintf(&id_str, 0, "%s:%d", STR_PTR(purl->host), purl->port ? purl->port : 80);
-		php_persistent_handle_factory_t *pf = php_persistent_handle_concede(NULL, ZEND_STRL("http\\Client\\Curl\\Request"), id_str, id_len, NULL, NULL TSRMLS_CC);
-
-		if (pf) {
-			rf = php_resource_factory_init(NULL, php_persistent_handle_get_resource_factory_ops(), pf, (void (*)(void*)) php_persistent_handle_abandon);
-		}
-
-		php_url_free(purl);
-		efree(id_str);
-	}
-
-	if (!rf) {
 		rf = php_resource_factory_init(NULL, &php_http_curle_resource_factory_ops, NULL, NULL);
 	}
 
+	efree(id_str);
+
 	return rf;
 }
 
@@ -1894,6 +1883,17 @@ static void php_http_client_curl_reset(php_http_client_t *h)
 	}
 }
 
+static inline void php_http_client_curl_get_timeout(php_http_client_curl_t *curl, long max_tout, struct timeval *timeout)
+{
+	if ((CURLM_OK == curl_multi_timeout(curl->handle, &max_tout)) && (max_tout > 0)) {
+		timeout->tv_sec = max_tout / 1000;
+		timeout->tv_usec = (max_tout % 1000) * 1000;
+	} else {
+		timeout->tv_sec = 0;
+		timeout->tv_usec = 1000;
+	}
+}
+
 #ifdef PHP_WIN32
 #	define SELECT_ERROR SOCKET_ERROR
 #else
@@ -1909,10 +1909,18 @@ static STATUS php_http_client_curl_wait(php_http_client_t *h, struct timeval *cu
 
 #if PHP_HTTP_HAVE_EVENT
 	if (curl->useevents) {
-		TSRMLS_FETCH_FROM_CTX(h->ts);
+		if (!event_initialized(curl->timeout)) {
+			event_assign(curl->timeout, curl->evbase, CURL_SOCKET_TIMEOUT, 0, php_http_curlm_timeout_callback, h);
+		} else if (custom_timeout && timerisset(custom_timeout)) {
+			event_add(curl->timeout, custom_timeout);
+		} else if (!event_pending(curl->timeout, EV_TIMEOUT, NULL)) {
+			php_http_client_curl_get_timeout(curl, 1000, &timeout);
+			event_add(curl->timeout, &timeout);
+		}
 
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "not implemented");
-		return FAILURE;
+		event_base_loop(curl->evbase, EVLOOP_ONCE);
+
+		return SUCCESS;
 	}
 #endif
 
@@ -1924,15 +1932,7 @@ static STATUS php_http_client_curl_wait(php_http_client_t *h, struct timeval *cu
 		if (custom_timeout && timerisset(custom_timeout)) {
 			timeout = *custom_timeout;
 		} else {
-			long max_tout = 1000;
-
-			if ((CURLM_OK == curl_multi_timeout(curl->handle, &max_tout)) && (max_tout > 0)) {
-				timeout.tv_sec = max_tout / 1000;
-				timeout.tv_usec = (max_tout % 1000) * 1000;
-			} else {
-				timeout.tv_sec = 0;
-				timeout.tv_usec = 1000;
-			}
+			php_http_client_curl_get_timeout(curl, 1000, &timeout);
 		}
 
 		if (MAX == -1) {
@@ -1951,12 +1951,9 @@ static int php_http_client_curl_once(php_http_client_t *h)
 
 #if PHP_HTTP_HAVE_EVENT
 	if (curl->useevents) {
-		TSRMLS_FETCH_FROM_CTX(h->ts);
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "not implemented");
-		return FAILURE;
-	}
+		event_base_loop(curl->evbase, EVLOOP_NONBLOCK);
+	} else
 #endif
-
 	while (CURLM_CALL_MULTI_PERFORM == curl_multi_perform(curl->handle, &curl->unfinished));
 
 	php_http_curlm_responsehandler(h);
